import * as THREE from '../build/three.module.js';
import { GUI } from './jsm/libs/dat.gui.module.js';
import { OrbitControls } from './jsm/controls/OrbitControls.js';
import { Water } from './jsm/objects/Water2.js';

class App {
    constructor() {
        this._initialize();
    }

    _initialize() {
        this.domWebGL = document.createElement('div');
        document.body.appendChild(this.domWebGL);

        let renderer = new THREE.WebGLRenderer({ antialias: true });
        this.domWebGL.appendChild(renderer.domElement);

        this.renderer = renderer;

        this.scene = new THREE.Scene();
        this._clock = new THREE.Clock();

        this._setupCamera();
        this._setupGUI();
        this._setupControls();
        this._setupLights();
        this._setupModel();

        window.onresize = this.resize.bind(this);

        this.resize();
    }

    _setupCamera() {
        const camera = new THREE.PerspectiveCamera(45, window.innerWidth / window.innerHeight, 0.1, 200);
        camera.position.set(-15, 7, 15);
        camera.lookAt(this.scene.position);

        this._camera = camera;
    }

    _setupControls() {
        const controls = new OrbitControls(this._camera, this.renderer.domElement);
        controls.minDistance = 5;
        controls.maxDistance = 50;

        this._controls = controls;
    }

    _setupLights() {
        const ambientLight = new THREE.AmbientLight(0xcccccc, 0.4);
        this.scene.add(ambientLight);

        const directionLight = new THREE.DirectionalLight(0xffffff, 0.6);
        directionLight.position.set(-1, 1, 1);
        this.scene.add(directionLight);
    }

    _setupModel() {
        const torusKnotGeometry = new THREE.TorusKnotGeometry(3, 1, 256, 32);
        const torusKnotMaterial = new THREE.MeshNormalMaterial();
        const torusKnot = new THREE.Mesh(torusKnotGeometry, torusKnotMaterial);
        torusKnot.position.y = 2;
        torusKnot.scale.set(0.5, 0.5, 0.5);
        this.scene.add(torusKnot);

        const groundGeometry = new THREE.PlaneGeometry(20, 20);
        const groundMaterial = new THREE.MeshStandardMaterial({ roughness: 0.8, metalness: 0.4 });
        const ground = new THREE.Mesh(groundGeometry, groundMaterial);
        ground.rotation.x = Math.PI * (-0.5);
        this.scene.add(ground);

        const textureLoader = new THREE.TextureLoader();
        textureLoader.load("textures/hardwood2_diffuse.jpg", (map) => {
            map.wrapS = THREE.RepeatWrapping;
            map.wrapT = THREE.RepeatWrapping;
            map.anisotropy = 16;
            map.repeat.set(4, 4);
            groundMaterial.map = map;
            groundMaterial.needsUpdate = true;
        });

        const waterGeometry = new THREE.PlaneGeometry(20, 20);
        const water = new Water(waterGeometry, {
            color: this._params.color,
            scale: this._params.scale,
            flowDirection: new THREE.Vector2(this._params.flowX, this._params.flowY),
            textureWidth: 1024,
            textureHeight: 1024
        });
        water.position.y = 1;
        water.rotation.x = -Math.PI * 0.5;
        this.scene.add(water);

        const cubeTextureLoader = new THREE.CubeTextureLoader();
        cubeTextureLoader.setPath("textures/cube/Park2/");
        const cubeTexture = cubeTextureLoader.load([
            "posx.jpg", "negx.jpg", "posy.jpg", "negy.jpg", "posz.jpg", "negz.jpg" 
        ]);
         this.scene.background = cubeTexture;

        this._torusKnot = torusKnot;
        this._water = water;
    }


    update(time) {
        const delta = this._clock.getDelta();
        
        this._torusKnot.rotation.x += delta;
        this._torusKnot.rotation.y += delta * 0.5;

        this._controls.update();
    }

    render(time) {
        requestAnimationFrame(this.render.bind(this));

        this.renderer.render(this.scene, this._camera);
        this.update(time);
    }

    resize() {
        this._camera.aspect = window.innerWidth / window.innerHeight;
        this._camera.updateProjectionMatrix();

        this.renderer.setPixelRatio(window.devicePixelRatio);
        this.renderer.setSize(window.innerWidth, window.innerHeight);
    }

    _setupGUI() {
        const params = {
            color: "#ffffff",
            scale: 4,
            flowX: 1,
            flowY: 1
        };

        const gui = new GUI();

        gui.addColor(params, "color").onChange((value) => {
            this._water.material.uniforms["color"].value.set(value);
        });

        gui.add(params, "scale", 1, 10).onChange((value) => {
            this._water.material.uniforms["config"].value.w = value;
        });

        gui.add(params, "flowX", -1, 1).step(0.01).onChange((value) => {
            this._water.material.uniforms["flowDirection"].value.x = value;
            this._water.material.uniforms["flowDirection"].value.normalize();
        });

        gui.add(params, "flowY", -1, 1).step(0.01).onChange((value) => {
            this._water.material.uniforms["flowDirection"].value.y = value;
            this._water.material.uniforms["flowDirection"].value.normalize();
        });

        gui.open();

        this._params = params;
    }
}

window.onload = function () {
    (new App()).render();
}